@@ -2,20 +2,28 @@
    Copyright (c) 2006, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+   along with this program; if not, write to the Free Software Foundation,
+   51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
 */
 
 #include "client_priv.h"
+#include "my_default.h"
 #include <sslopt-vars.h>
 #include "../scripts/mysql_fix_privilege_tables_sql.c"
 
@@ -105,7 +113,7 @@ static struct my_option my_long_options[]=
   {"password", 'p',
    "Password to use when connecting to server. If password is not given,"
    " it's solicited on the tty.", &opt_password,&opt_password,
-   0, GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
+   0, GET_PASSWORD, OPT_ARG, 0, 0, 0, 0, 0, 0},
 #ifdef __WIN__
   {"pipe", 'W', "Use named pipes to connect to server.", 0, 0, 0,
    GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
@@ -183,6 +191,7 @@ static void die(const char *fmt, ...)
   }
   va_end(args);
 
+  DBUG_LEAVE;
   free_used_memory();
   my_end(my_end_arg);
   exit(1);
@@ -226,6 +235,7 @@ static void add_one_option(DYNAMIC_STRING* ds,
     eq= "=";
     switch (opt->var_type & GET_TYPE_MASK) {
     case GET_STR:
+    case GET_PASSWORD:
       arg= argument;
       break;
     case GET_BOOL:
@@ -334,8 +344,16 @@ static int run_command(char* cmd,
   FILE *res_file;
   int error;
 
+  if (! ds_res)
+  {
+    fflush(stdout);
+    fflush(stderr);
+  }
   if (!(res_file= popen(cmd, "r")))
-    die("popen(\"%s\", \"r\") failed", cmd);
+  {
+    die("popen(\"%s\", \"r\") failed, error: %s (errno = %d)", cmd,
+        strerror(errno), errno);
+  }
 
   while (fgets(buf, sizeof(buf), res_file))
   {
@@ -352,8 +370,39 @@ static int run_command(char* cmd,
     }
   }
 
+  if (! ds_res)
+  {
+    fflush(stdout);
+    fflush(stderr);
+  }
+
+  if (ferror(res_file))
+  {
+    fprintf(stderr,
+            "%s: fgets from popen(\"%s\", \"r\") failed, error: %s (errno = %d)",
+            my_progname, cmd, strerror(errno), errno);
+  }
+
   error= pclose(res_file);
-  return WEXITSTATUS(error);
+  if (error == -1)
+  {
+    fprintf(stderr, "%s: pclose for popen(\"%s\", \"r\") failed, error: %s "
+            "(errno = %d)\n",
+            my_progname, cmd, strerror(errno), errno);
+    return -1;
+  }
+  if (WIFEXITED(error))
+    return WEXITSTATUS(error);
+  if (WIFSIGNALED(error)) {
+    fprintf(stderr, "%s: child process of popen(\"%s\", \"r\") terminated "
+            "with signal %d\n", my_progname, cmd, WTERMSIG(error));
+    return -1;
+  }
+  else {
+    fprintf(stderr, "%s: child process of popen(\"%s\", \"r\") unknown "
+            "termination status %d\n", my_progname, cmd, error);
+    return -1;
+  }
 }
 
 
@@ -415,6 +464,7 @@ static void find_tool(char *tool_executable_name, const char *tool_name,
 {
   char *last_fn_libchar;
   DYNAMIC_STRING ds_tmp;
+  int error;
   DBUG_ENTER("find_tool");
   DBUG_PRINT("enter", ("progname: %s", my_progname));
 
@@ -467,14 +517,15 @@ static void find_tool(char *tool_executable_name, const char *tool_name,
   /*
     Make sure it can be executed
   */
-  if (run_tool(tool_executable_name,
+  error= run_tool(tool_executable_name,
                &ds_tmp, /* Get output from command, discard*/
                "--no-defaults",
                "--help",
                "2>&1",
                IF_WIN("> NUL", "> /dev/null"),
-               NULL))
-    die("Can't execute '%s'", tool_executable_name);
+               NULL);
+  if (error)
+    die("Can't execute '%s', returned %d", tool_executable_name, error);
 
   dynstr_free(&ds_tmp);
 
@@ -542,6 +593,13 @@ static int run_query(const char *query, DYNAMIC_STRING *ds_res,
   my_close(fd, MYF(0));
   my_delete(query_file_path, MYF(0));
 
+  if (ret)
+  {
+    fprintf(stderr,
+            "mysql_upgrade: running mysql with query \"%s\" returned %d\n",
+            query, ret);
+  }
+
   DBUG_RETURN(ret);
 }
 
@@ -567,7 +625,7 @@ static int extract_variable_from_show(DYNAMIC_STRING* ds, char* value)
   if ((value_end= strchr(value_start, '\n')) == NULL)
     return 1; /* Unexpected result */
 
-  len= (size_t) min(FN_REFLEN, value_end-value_start);
+  len= (size_t) MY_MIN(FN_REFLEN, value_end-value_start);
   strncpy(value, value_start, len);
   value[len]= '\0';
   return 0;
@@ -631,7 +689,7 @@ static int upgrade_already_done(void)
     Read from file, don't care if it fails since it
     will be detected by the strncmp
   */
-  bzero(buf, sizeof(buf));
+  memset(buf, 0, sizeof(buf));
   res= fgets(buf, sizeof(buf), in);
 
   my_fclose(in, MYF(0));
@@ -717,6 +775,7 @@ static int run_mysqlcheck_upgrade(void)
                   ds_args.str,
                   "--check-upgrade",
                   "--all-databases",
+                  "--skip-database=mysql",
                   "--auto-repair",
                   opt_write_binlog ? "--write-binlog" : "--skip-write-binlog",
                   NULL);
@@ -731,13 +790,45 @@ static int run_mysqlcheck_fixnames(void)
                   "--no-defaults",
                   ds_args.str,
                   "--all-databases",
+                  "--skip-database=mysql",
                   "--fix-db-names",
                   "--fix-table-names",
                   opt_write_binlog ? "--write-binlog" : "--skip-write-binlog",
                   NULL);
 }
 
+/** performs the same operation as mysqlcheck_upgrade, but on the mysql db */
+static int run_mysqlcheck_mysql_db_upgrade(void)
+{
+  print_conn_args("mysqlcheck");
+  return run_tool(mysqlcheck_path,
+                  NULL, /* Send output from mysqlcheck directly to screen */
+                  "--no-defaults",
+                  ds_args.str,
+                  "--check-upgrade",
+                  "--databases",
+                  "--auto-repair",
+                  opt_write_binlog ? "--write-binlog" : "--skip-write-binlog",
+                  "mysql",
+                  NULL);
+}
 
+
+/** performs the same operation as mysqlcheck_fixnames, but on the mysql db */
+static int run_mysqlcheck_mysql_db_fixnames(void)
+{
+  print_conn_args("mysqlcheck");
+  return run_tool(mysqlcheck_path,
+                  NULL, /* Send output from mysqlcheck directly to screen */
+                  "--no-defaults",
+                  ds_args.str,
+                  "--databases",
+                  "--fix-db-names",
+                  "--fix-table-names",
+                  opt_write_binlog ? "--write-binlog" : "--skip-write-binlog",
+                  "mysql",
+                  NULL);
+}
 static const char *expected_errors[]=
 {
   "ERROR 1060", /* Duplicate column name */
@@ -796,17 +887,35 @@ static void print_line(char* line)
 static int run_sql_fix_privilege_tables(void)
 {
   int found_real_errors= 0;
+  const char **query_ptr;
+  DYNAMIC_STRING ds_script;
   DYNAMIC_STRING ds_result;
   DBUG_ENTER("run_sql_fix_privilege_tables");
 
+  if (init_dynamic_string(&ds_script, "", 65536, 1024))
+    die("Out of memory");
+
   if (init_dynamic_string(&ds_result, "", 512, 512))
     die("Out of memory");
 
   verbose("Running 'mysql_fix_privilege_tables'...");
-  run_query(mysql_fix_privilege_tables,
+
+  /*
+    Individual queries can not be executed independently by invoking
+    a forked mysql client, because the script uses session variables
+    and prepared statements.
+  */
+  for ( query_ptr= &mysql_fix_privilege_tables[0];
+        *query_ptr != NULL;
+        query_ptr++
+      )
+  {
+    dynstr_append(&ds_script, *query_ptr);
+  }
+
+  run_query(ds_script.str,
             &ds_result, /* Collect result */
             TRUE);
-
   {
     /*
       Scan each line of the result for real errors
@@ -823,7 +932,8 @@ static int run_sql_fix_privilege_tables(void)
         found_real_errors++;
         print_line(line);
       }
-      else if (strncmp(line, "WARNING", 7) == 0)
+      else if ((strncmp(line, "WARNING", 7) == 0) ||
+               (strncmp(line, "Warning", 7) == 0))
       {
         print_line(line);
       }
@@ -831,6 +941,7 @@ static int run_sql_fix_privilege_tables(void)
   }
 
   dynstr_free(&ds_result);
+  dynstr_free(&ds_script);
   DBUG_RETURN(found_real_errors);
 }
 
@@ -869,7 +980,6 @@ static int check_version_match(void)
   if (init_dynamic_string(&ds_version, NULL, NAME_CHAR_LEN, NAME_CHAR_LEN))
     die("Out of memory");
 
-
   if (run_query("show variables like 'version'", &ds_version, FALSE))
   {
     fprintf(stderr, "Error: Failed while fetching Server version! Could be"
@@ -901,7 +1011,7 @@ static int check_version_match(void)
 
 int main(int argc, char **argv)
 {
-  char self_name[FN_REFLEN];
+  char self_name[FN_REFLEN + 1];
 
   MY_INIT(argv[0]);
 
@@ -916,8 +1026,10 @@ int main(int argc, char **argv)
       init_dynamic_string(&conn_args, "", 512, 256))
     die("Out of memory");
 
+  my_getopt_use_args_separator= TRUE;
   if (load_defaults("my", load_default_groups, &argc, &argv))
     die(NULL);
+  my_getopt_use_args_separator= FALSE;
   defaults_argv= argv; /* Must be freed by 'free_defaults' */
 
   if (handle_options(&argc, &argv, my_long_options, get_one_option))
@@ -968,22 +1080,42 @@ int main(int argc, char **argv)
 
   /*
     Run "mysqlcheck" and "mysql_fix_privilege_tables.sql"
+    First run mysqlcheck on the system database.
+    Then do the upgrade.
+    And then run mysqlcheck on all tables.
   */
   if (!opt_systables_only)
   {
-    if (run_mysqlcheck_fixnames())
-      die("Error during call to mysql_check for fixing the db/tables names.");
-
-    if (run_mysqlcheck_upgrade())
-      die("Error during call to mysql_check for upgrading the tables names.");
+    if (run_mysqlcheck_mysql_db_fixnames())
+    {
+      die("Error during call to mysql_check for fixing the db/tables names on "
+          "mysql db");
+    }
+    if (run_mysqlcheck_mysql_db_upgrade())
+    {
+      die("Error during call to mysql_check for upgrading the tables names on "
+          "mysql db");
+    }
   }
-
   if (run_sql_fix_privilege_tables())
   {
     /* Specific error msg (if present) would be printed in the function call
      * above */
     die("Upgrade failed");
   }
+  if (!opt_systables_only)
+  {
+    if (run_mysqlcheck_fixnames())
+    {
+      die("Error during call to mysql_check for fixing the db/tables names on "
+          "all db(s) except mysql");
+    }
+    if (run_mysqlcheck_upgrade())
+    {
+      die("Error during call to mysql_check for upgrading the tables names on "
+          "all db(s) except mysql");
+    }
+  }
   verbose("OK");
 
   /* Create a file indicating upgrade has been performed */
