@@ -36,7 +36,7 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 #define BACKUP_MANAGER_H
 
 #ident "Copyright (c) 2012-2013 Tokutek Inc.  All rights reserved."
-#ident "$Id: 18af855f6b03f481beb56813007c73c437ab6f67 $"
+#ident "$Id: c12929525b149aa679ae4d7a4b6c5d49a7869c80 $"
 
 #include "backup.h"
 #include "backup_directory.h"
@@ -49,20 +49,20 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 #include <stdarg.h>
 #include <sys/types.h>
 #include <vector>
+#include <atomic>
 
 class manager : public manager_state
 {
 private:
 
 #ifdef GLASSBOX
-    volatile bool m_pause_disable;
-    volatile bool m_start_copying; // For test purposes, we can arrange to initialize the backup but not actually start copying.
-    volatile bool m_keep_capturing; // For test purposes, we can arrange to keep capturing the backup until the client tells us to stop.
-    volatile bool m_is_capturing;   // Backup manager sets to true when capturing is running, sets to false when capturing has stopped.   We look at m_start_copying after setting m_is_capturing=true.
-    volatile bool m_done_copying;   // Backup manager sets this true when copying is done.  Happens after m_is_captring
+    std::atomic_bool m_pause_disable;
+    std::atomic_bool m_start_copying; // For test purposes, we can arrange to initialize the backup but not actually start copying.
+    std::atomic_bool m_keep_capturing; // For test purposes, we can arrange to keep capturing the backup until the client tells us to stop.
+    std::atomic_bool m_is_capturing;   // Backup manager sets to true when capturing is running, sets to false when capturing has stopped.   We look at m_start_copying after setting m_is_capturing=true.
+    std::atomic_bool m_done_copying;   // Backup manager sets this true when copying is done.  Happens after m_is_captring
 #endif
 
-    volatile bool m_is_dead; // true if some error occured so that the backup system shouldn't try any more.
     volatile bool m_backup_is_running; // true if the backup is running.  This can be accessed without any locks.
 
     fmap m_map;
@@ -75,7 +75,7 @@ private:
     backup_session *m_session;
     static pthread_rwlock_t m_session_rwlock;
 
-    volatile unsigned long m_throttle;
+    std::atomic_ulong m_throttle;
 
     // Error handling.
     static pthread_mutex_t m_error_mutex;     // When testing errors grab this mutex. 
@@ -116,7 +116,7 @@ public:
     //  If this function is called on the backup thread, the errors are reported immediately.
     //  If this function is called on another thread, the error is saved for later so that the backup thread can report it.
   private:
-    void backup_error_ap(int errnum, const char *format, va_list ap) throw(); // This is the internal shared part of those two functions.
+    void backup_error_ap(int errnum, const char *format, va_list ap) throw() __attribute((format(printf,3,0))); // This is the internal shared part of those two functions.
 
   public:
     // TODO: #6537 Factor the test interface out of the main class, cleanly.
@@ -137,7 +137,7 @@ private:
     void exit_session_and_unlock_or_die(void) throw();
     int prepare_directories_for_backup(backup_session *session, const backtrace bt) throw();
     void disable_descriptions(void) throw();
-    void set_error_internal(int errnum, const char *format, va_list ap) throw();
+    void set_error_internal(int errnum, const char *format, va_list ap) throw() __attribute__((format(printf,3,0)));
     int setup_description_and_source_file(int fd, const char *file, const int flags) throw();
     bool should_capture_unlink_of_file(const char *file) throw();
     friend class with_manager_enter_session_and_lock;
