@@ -373,8 +373,6 @@ mysqld_ld_preload_text() {
 
 # set_malloc_lib LIB
 # - If LIB is empty, do nothing and return
-# - If LIB is 'tcmalloc', look for tcmalloc shared library in $malloc_dirs.
-#   tcmalloc is part of the Google perftools project.
 # - If LIB is an absolute path, assume it is a malloc shared library
 #
 # Put LIB in mysqld_ld_preload, which will be added to LD_PRELOAD when
@@ -384,24 +382,6 @@ set_malloc_lib() {
   malloc_dirs="/usr/lib /usr/lib64 /usr/lib/i386-linux-gnu /usr/lib/x86_64-linux-gnu"
   malloc_lib="$1"
 
-  if [ "$malloc_lib" = tcmalloc ]; then
-    malloc_lib=
-    for libdir in `echo $malloc_dirs`; do
-      for flavor in _minimal '' _and_profiler _debug; do
-        tmp="$libdir/libtcmalloc$flavor.so"
-        #log_notice "DEBUG: Checking for malloc lib '$tmp'"
-        [ -r "$tmp" ] || continue
-        malloc_lib="$tmp"
-        break 2
-      done
-    done
-
-    if [ -z "$malloc_lib" ]; then
-      log_error "no shared library for --malloc-lib=tcmalloc found in $malloc_dirs"
-      exit 1
-    fi
-  fi
-
   # Allow --malloc-lib='' to override other settings
   [ -z  "$malloc_lib" ] && return
 
@@ -425,8 +405,7 @@ set_malloc_lib() {
       esac
       ;;
     *)
-      log_error "--malloc-lib must be an absolute path or 'tcmalloc'; " \
-        "ignoring value '$malloc_lib'"
+      log_error "--malloc-lib must be an absolute path ignoring value '$malloc_lib'"
       exit 1
       ;;
   esac
