@@ -1,17 +1,24 @@
 /* Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   along with this program; if not, write to the Free Software Foundation,
+   51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA */
 
 
 /**
@@ -36,12 +43,17 @@
 #include "sql_show.h"                     // schema_table_store_record
 #include "sql_class.h"                    // THD
 
+#include <algorithm>
+
+using std::min;
+using std::max;
+
 #define TIME_FLOAT_DIGITS 9
 /** two vals encoded: (dec*100)+len */
 #define TIME_I_S_DECIMAL_SIZE (TIME_FLOAT_DIGITS*100)+(TIME_FLOAT_DIGITS-3)
 
-#define MAX_QUERY_LENGTH 300
-#define MAX_QUERY_HISTORY 101
+#define MAX_QUERY_LENGTH 300U
+#define MAX_QUERY_HISTORY 101U
 
 /**
   Connects Information_Schema and Profiling.
@@ -120,9 +132,7 @@ int make_profile_table_for_show(THD *thd, ST_SCHEMA_TABLE *schema_table)
                                       NullS, NullS, field_info->field_name);
     if (field)
     {
-      field->set_name(field_info->old_name,
-                      (uint) strlen(field_info->old_name),
-                      system_charset_info);
+      field->item_name.copy(field_info->old_name);
       if (add_item_to_list(thd, field))
         return 1;
     }
@@ -535,7 +545,7 @@ void PROFILING::set_query_source(char *query_source_arg, uint query_length_arg)
   DBUG_VOID_RETURN;
 }
 
-bool PROFILING::enabled_getrusage()
+bool PROFILING::enabled_getrusage() const
 {
   return ((thd->variables.log_slow_verbosity & (ULL(1) << SLOG_V_PROFILING_USE_GETRUSAGE)) != 0);
 }
@@ -550,7 +560,8 @@ static void my_b_print_status(IO_CACHE *log_file, const char *status,
                               PROF_MEASUREMENT *start, PROF_MEASUREMENT *stop)
 {
   DBUG_ENTER("my_b_print_status");
-  DBUG_ASSERT(log_file != NULL && status != NULL);
+  DBUG_ASSERT(log_file != NULL);
+  DBUG_ASSERT(status != NULL);
   char query_time_buff[22+7];
   const char *tmp;
 
@@ -579,7 +590,7 @@ static void my_b_print_status(IO_CACHE *log_file, const char *status,
   Print output for current query to file 
 */
 
-int PROFILING::print_current(IO_CACHE *log_file)
+int PROFILING::print_current(IO_CACHE *log_file) const
 {
   DBUG_ENTER("PROFILING::print_current");
   ulonglong row_number= 0;
