@@ -1,13 +1,20 @@
-/* Copyright (c) 2004, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2004, 2016, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  it under the terms of the GNU General Public License, version 2.0,
+  as published by the Free Software Foundation.
+
+  This program is also distributed with certain software (including
+  but not limited to OpenSSL) that is licensed under separate terms,
+  as designated in a particular file or component or in included license
+  documentation.  The authors of MySQL hereby grant you an additional
+  permission to link the program and your derivative works with the
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  GNU General Public License, version 2.0, for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -41,10 +48,6 @@ TODO:
  -Brian
 */
 
-#ifdef USE_PRAGMA_IMPLEMENTATION
-#pragma implementation        // gcc: Class implementation
-#endif
-
 #include "my_global.h"
 #include "sql_priv.h"
 #include "sql_class.h"                          // SSV
@@ -53,6 +56,10 @@ TODO:
 #include "ha_tina.h"
 #include "probes_mysql.h"
 
+#include <algorithm>
+
+using std::min;
+using std::max;
 
 /*
   uchar + uchar + ulonglong + ulonglong + ulonglong + ulonglong + uchar
@@ -119,7 +126,7 @@ int sort_set (tina_set *a, tina_set *b)
 }
 
 static uchar* tina_get_key(TINA_SHARE *share, size_t *length,
-                          my_bool not_used __attribute__((unused)))
+                          my_bool not_used MY_ATTRIBUTE((unused)))
 {
   *length=share->table_name_length;
   return (uchar*) share->table_name;
@@ -150,14 +157,11 @@ static void init_tina_psi_keys(void)
   const char* category= "csv";
   int count;
 
-  if (PSI_server == NULL)
-    return;
-
   count= array_elements(all_tina_mutexes);
-  PSI_server->register_mutex(category, all_tina_mutexes, count);
+  mysql_mutex_register(category, all_tina_mutexes, count);
 
   count= array_elements(all_tina_files);
-  PSI_server->register_file(category, all_tina_files, count);
+  mysql_file_register(category, all_tina_files, count);
 }
 #endif /* HAVE_PSI_INTERFACE */
 
@@ -1020,9 +1024,6 @@ int ha_tina::write_row(uchar * buf)
 
   ha_statistic_increment(&SSV::ha_write_count);
 
-  if (table->timestamp_field_type & TIMESTAMP_AUTO_SET_ON_INSERT)
-    table->timestamp_field->set_time();
-
   size= encode_quote(buf);
 
   if (!share->tina_write_opened)
@@ -1071,8 +1072,8 @@ int ha_tina::open_update_temp_file_if_needed()
 
 /*
   This is called for an update.
-  Make sure you put in code to increment the auto increment, also
-  update any timestamp data. Currently auto increment is not being
+  Make sure you put in code to increment the auto increment.
+  Currently auto increment is not being
   fixed since autoincrements have yet to be added to this table handler.
   This will be called in a table scan right before the previous ::rnd_next()
   call.
@@ -1085,9 +1086,6 @@ int ha_tina::update_row(const uchar * old_data, uchar * new_data)
 
   ha_statistic_increment(&SSV::ha_update_count);
 
-  if (table->timestamp_field_type & TIMESTAMP_AUTO_SET_ON_UPDATE)
-    table->timestamp_field->set_time();
-
   size= encode_quote(new_data);
 
   /*
@@ -1350,8 +1348,7 @@ bool ha_tina::get_write_pos(my_off_t *end_pos, tina_set *closest_hole)
   if (closest_hole == chain_ptr) /* no more chains */
     *end_pos= file_buff->end();
   else
-    *end_pos= min(file_buff->end(),
-                  closest_hole->begin);
+    *end_pos= min(file_buff->end(), closest_hole->begin);
   return (closest_hole != chain_ptr) && (*end_pos == closest_hole->begin);
 }
 
@@ -1458,8 +1455,8 @@ int ha_tina::rnd_end()
 
     /* Open the file again */
     if ((data_file= mysql_file_open(csv_key_file_data,
-                                     share->data_file_name,
-                                     O_RDONLY, MYF(MY_WME))) == -1)
+                                    share->data_file_name,
+                                    O_RDONLY, MYF(MY_WME))) == -1)
       DBUG_RETURN(my_errno ? my_errno : -1);
     /*
       As we reopened the data file, increase share->data_file_version 
